@@ -1,9 +1,10 @@
 relation_violations <-
 function(x, family = c("T", "transitive",
-                         "S", "symmetric",
-                         "A", "antisymmetric",
-                         "C", "complete",
-                         "R", "reflexive"))
+                       "S", "symmetric",
+                       "A", "antisymmetric",
+                       "C", "complete",
+                       "M", "match",
+                       "R", "reflexive"))
 {
     if (!relation_is_endorelation(x))
         stop("Relation violations only defined for endorelations.")
@@ -16,6 +17,7 @@ function(x, family = c("T", "transitive",
            S =, symmetric = .non_symmetry(I),
            A =, antisymmetric = .non_antisymmetry(I),
            C =, complete = .non_completeness(I),
+           M =, match = .non_strictlycompleteness(I),
            R =, reflexive = .non_reflexivity(I)
            )
 }
@@ -23,26 +25,34 @@ function(x, family = c("T", "transitive",
 .non_transitivity <-
 function(I)
     sum(sapply(seq_len(nrow(I)),
-               function(j) outer(I[, j], I[j, ], "+") - I) > 1)
+               function(j) outer(I[, j], I[j, ], .T.) - I) > 1)
 
 .non_symmetry <-
 function(I)
-    sum(I & I != t(I))
+    sum(.T.(I, I != t(I)))
 
 .non_antisymmetry <-
 function(I)
 {
     diag(I) <- 0
-    sum(I & t(I)) / 2
+    sum(.T.(I, t(I))) / 2
 }
 
 .non_completeness <-
 function(I)
 {
-    I <- 1 - I
+    I <- .N.(I)
+    diag(I) <- 0
+    sum(.T.(I, t(I))) / 2
+}
+
+.non_strictlycompleteness <-
+function(I)
+{
+    I <- .N.(I)
     D <- diag(I)
     diag(I) <- 0
-    sum(I & t(I)) / 2 + sum(D)
+    sum(.T.(I, t(I))) / 2 + sum(D)
 }
 
 .non_reflexivity <-
